/* dio_structures.h
 *
 * Command structure definitions
 *
 * This file can be used by Lua FFI.
 *
 */


#pragma pack(push)
#pragma pack(1)

typedef enum { DIO_TS_OFFSET = 1, DIO_GET_SYNC_TS, DIO_GET_TS,DIO_SETUP_SYNC_TS,
               DIO_TXDR_POWER_ON_5V, DIO_TXDR_POWER_ON_12V, DIO_TXDR_POWER_OFF,
               DIO_HC_START, DIO_HC_STOP, DIO_HC_SET_THRESHOLDS,
               DIO_IC_START, DIO_IC_STOP,
               DIO_PC_START, DIO_PC_STOP,
               DIO_QC_START, DIO_QC_STOP,
               DIO_OS_START, DIO_OS_STOP,
               DIO_EVENT_SETUP, DIO_EVENT_START, DIO_EVENT_STOP,
               DIO_GET_PIN_STATE,
               DIO_SET_OUTPUT0_CLOSED, DIO_SET_OUTPUT0_OPEN,
               DIO_SET_OUTPUT1_CLOSED, DIO_SET_OUTPUT1_OPEN,
               DIO_GET_CABLE_SENSE,
               DIO_LAST_COMMAND = 0x7FFFFFFF    /* Insures 32-bit enum */
} dio_cmd_t;

typedef struct {
    int64_t                     start_time;
    cmd_rate_parameters_s       sample_rate;
    uint32_t                    phys_address;
    uint8_t                     swap_a_b;
    uint8_t                     invert_index;
    uint8_t                     invert_b;
    uint8_t                     invert_a;
    uint8_t                     count_down_b_fall_a_one;
    uint8_t                     count_down_b_rise_a_zero;
    uint8_t                     count_down_a_fall_b_zero;
    uint8_t                     count_down_a_rise_b_one;
    uint8_t                     count_up_b_fall_a_zero;
    uint8_t                     count_up_b_rise_a_one;
    uint8_t                     count_up_a_fall_b_one;
    uint8_t                     count_up_a_rise_b_zero;
} dio_cmd_qc_start_s;

typedef struct {
    int64_t                     start_time;
    cmd_rate_parameters_s       sample_rate;
    uint32_t                    phys_address;
    uint32_t                    decay_rate;
    uint32_t                    dynamic_mode;
    uint32_t                    decay_divider;
    uint32_t                    dac_update_divider;
    uint32_t                    threshold_ratio_lo2hi;
    uint32_t                    threshold_ratio_hi2lo;
    uint32_t                    threshold_hi;
    uint32_t                    threshold_lo;
} dio_cmd_hc_start_s;

typedef struct {
    uint32_t                    threshold_hi;
    uint32_t                    threshold_lo;
} dio_cmd_hc_set_th_s;

typedef struct {
    int64_t                     start_time;
    cmd_rate_parameters_s       sample_rate;
    uint32_t                    phys_address;
} dio_cmd_ic_start_s;

typedef struct {
    int64_t                     start_time;
    cmd_rate_parameters_s       sample_rate;
    uint32_t                    phys_address;
} dio_cmd_pc_start_s;

typedef struct {
    uint8_t         cable_unplugged;
    uint8_t         cable_plugged;
    uint8_t         index_mark_fall;
    uint8_t         sync_input0_fall;
    uint8_t         sync_input1_fall;
    uint8_t         sync_input2_fall;
    uint8_t         index_mark_rise;
    uint8_t         sync_input0_rise;
    uint8_t         sync_input1_rise;
    uint8_t         sync_input2_rise;
    uint8_t         match_closed1;
    uint8_t         match_open1;
    uint8_t         force_closed1;
    uint8_t         force_open1;
    uint8_t         match_closed0;
    uint8_t         match_open0;
    uint8_t         force_closed0;
    uint8_t         force_open0;
    uint8_t         reserved[2];
} dio_cmd_event_vector_s;

typedef struct {
    uint32_t                    output_bit;
    uint32_t                    phys_address;
} dio_cmd_os_start_s;

typedef struct {
    dio_cmd_t       cmd;
    uint32_t        cmd_id;
    union {
        cmd_ts_offset_s         ts_offset;
        dio_cmd_hc_set_th_s     threshold_setup;
        dio_cmd_qc_start_s      qc_start;
        dio_cmd_hc_start_s      hc_start;
        dio_cmd_ic_start_s      ic_start;
        dio_cmd_pc_start_s      pc_start;
        cmd_event_start_s       ev_start;
        dio_cmd_event_vector_s  events;
        dio_cmd_os_start_s      os_start;
        uint32_t                stop_channel;
    } prm;
} dio_cmd_s;

typedef struct {
    int64_t quadrature_count;
    int64_t latched_index_count;
    uint32_t direction_and_latch_status;
} qc_buffer_t;

typedef struct {
    uint16_t lo_peak;
    uint16_t hi_peak;
    uint16_t threshold_lo;
    uint16_t threshold_hi;
    uint32_t adc_data;
} hc_buffer_t;

typedef struct {
    uint32_t pin_state;
} ic_buffer_t;

typedef struct {
    uint32_t period_count_lo;
    uint32_t period_count_hi;
    uint32_t avg_period_count_lo;
    uint32_t avg_period_count_hi;
    uint32_t edge_count_lo;
    uint32_t edge_count_hi;
    uint32_t partial_period_count_lo;
    uint32_t partial_period_count_hi;
} pc_buffer_t;

typedef struct {
    int64_t     ts;
    uint32_t    match_closed;       /* 0 = open, 1 = closed */
} os_buffer_t;

typedef struct {
    int64_t                     ts;
    dio_cmd_event_vector_s      events;
} dio_ev_buffer_t;

#pragma pack(pop)

