local ffi = require 'ffi'
local lj_include = require 'lj_include'

ffi.cdef[[
struct sched_param {
    int __sched_priority;
};
int clone(int (*__fn) (void *__arg), void *__child_stack,
    int __flags, void *__arg, ...);
int unshare(int __flags);
int sched_getcpu(void);
struct __sched_param {
    int __sched_priority;
};
typedef unsigned long int __cpu_mask;
typedef struct {
    __cpu_mask __bits[1024 / (8 * sizeof(__cpu_mask))];
}
cpu_set_t;
int __sched_cpucount(size_t __setsize, const cpu_set_t * __setp);
cpu_set_t *__sched_cpualloc(size_t __count);
void __sched_cpufree(cpu_set_t * __set);
]]

local defs = {
    SCHED_OTHER = '0',
    SCHED_FIFO = '1',
    SCHED_RR = '2',
    SCHED_BATCH = '3',
    CSIGNAL = '0x000000ff',
    CLONE_VM = '0x00000100',
    CLONE_FS = '0x00000200',
    CLONE_FILES = '0x00000400',
    CLONE_SIGHAND = '0x00000800',
    CLONE_PTRACE = '0x00002000',
    CLONE_VFORK = '0x00004000',
    CLONE_PARENT = '0x00008000',
    CLONE_THREAD = '0x00010000',
    CLONE_NEWNS = '0x00020000',
    CLONE_SYSVSEM = '0x00040000',
    CLONE_SETTLS = '0x00080000',
    CLONE_PARENT_SETTID = '0x00100000',
    CLONE_CHILD_CLEARTID = '0x00200000',
    CLONE_DETACHED = '0x00400000',
    CLONE_UNTRACED = '0x00800000',
    CLONE_CHILD_SETTID = '0x01000000',
    CLONE_NEWUTS = '0x04000000',
    CLONE_NEWIPC = '0x08000000',
    CLONE_NEWUSER = '0x10000000',
    CLONE_NEWPID = '0x20000000',
    CLONE_NEWNET = '0x40000000',
    CLONE_IO = '0x80000000',
}
local format, cdef, C = string.format, ffi.cdef, ffi.C
for sym, val in pairs(defs) do
    if not pcall(function() return C[sym] end) then
        cdef(format("enum {%s = %s};", sym, val))
    end
end

