local m = {}
m.common = {}
m.MX1601B = require('mx.mx1601b').translator
m.MX1615B = require('mx.mx1615b').translator
m.MX1609B = require('mx.mx1609b').translator
--m.MX411B = require('mx.mx411b').translator
--m.MX471B = require('mx.mx471b').translator
--m.MX840B = require('mx.mx840b').translator

-- Translator functions used to translate parameters to/from setup info
-- to/from what is needed/supplied by the MX devices
local function translate(map, p, fromxml)
    p = tostring(p)
    if fromxml then
        if map[p] then
            if type(map[p]) == 'string' then
                return map[p]
            elseif type(map[p]) == 'table' then
                return map[p][1]
            end
        end
    end
    for k, v in pairs(map) do
        if type(v) == 'string' then
            if v == tostring(p) then return k end
        elseif type(v) == 'table' then
            for _, v2 in ipairs(v) do
                if v2 == tostring(p) then return k end
            end
        end
    end
end

function m.translate_function()
    return translate
end

function m.common:digital_filter_type(p, fromxml)
    return translate(
        {
            ['0'] = 'none',
            ['1'] = 'bessel_iir',
            ['2'] = 'butterworth_iir',
            ['3'] = 'linear_phase_fir',
            ['4'] = 'butterworth_fir',
        }, p, fromxml)
end

function m.common:calibration_rezero_mode(p, fromxml)
    if fromxml then
        if p ~= '1' then
            return 'all_test_runs'
        end
        return 'not_applicable'
    else
        if p ~= 'not_applicable' then
            return '0'
        end
        return '1'
    end
end


function m.common:sensor_input_mode(p, fromxml)
    return translate(
        {
            Voltage = 'volt',
            Bridge = 'bridge',
            IEPE = 'iepe',
            Resistor = 'resistor',
            Current = '4-20ma',
            Thermocouple = 'thermocouple',
            Pt = 'pt',
            Potentiometer = 'potentiometer',
            LVDT = 'lvdt',
            IndBridge = 'inductive_bridge',
            SSI = 'ssi_encoder_counter',
            Counter = {'pulse_counter', 'encoder_counter'},
            Frequency = {'pulse_frequency', 'encoder_frequency'},
        }, p, fromxml)
end

function m.common:sensor_input_min(p, fromxml)
    local v = tonumber(p)
    if not fromxml then
        if v then
            if v < 0 then return tostring(v * -1) end
        end
    else
        if v then
            if v > 0 then return tostring(v * -1) end
        end
    end
    return p
end

function m.common:calibration_mode(p, fromxml)
    return translate(
        {
            ZeroSpan = 'defined_zero_span',
            LinTable = 'defined_two_point',
            Gain = 'defined_slope_intercept',
            Straingage = 'strain_gage',
            Internal = 'internal',
        }, p, fromxml)
end


function m.common:units(p, fromxml)
    return translate(
        {
            ['100'] = "rad",
            ['101'] = "radian",
            ['102'] = "°",
            ['103'] = "%degrees",
            ['300'] = "m",
            ['301'] = "µm",
            ['302'] = "mm",
            ['303'] = "cm",
            ['304'] = "dm",
            ['305'] = "km",
            ['306'] = "inch",
            ['307'] = "feet",
            ['308'] = "yard",
            ['309'] = "mile",
            ['400'] = "kg",
            ['401'] = "g",
            ['402'] = "t",
            ['403'] = "kt",
            ['404'] = "tons",
            ['405'] = "lbs",
            ['500'] = "s",
            ['501'] = "ms",
            ['502'] = "µs",
            ['503'] = "min",
            ['504'] = "h",
            ['505'] = "days",
            ['600'] = "A",
            ['601'] = "A rms",
            ['602'] = "mA",
            ['603'] = "µA",
            ['604'] = "mA rms",
            ['605'] = "µA rms",
            ['700'] = "K",
            ['701'] = "°C",
            ['702'] = "°F",
            ['703'] = "°Rank",
            ['704'] = "°R",
            ['1000'] = "V/V",
            ['1001'] = "mV/V",
            ['1002'] = "µV/V",
            ['1100'] = "V",
            ['1101'] = "mV",
            ['1102'] = "µV",
            ['1103'] = "V rms",
            ['1104'] = "mV rms",
            ['1105'] = "µV rms",
            ['1200'] = "Ohm",
            ['1201'] = "kOhm",
            ['1202'] = "MOhm",
            ['1203'] = "mOhm",
            ['1300'] = "H",
            ['1301'] = "mH",
            ['1302'] = "µH",
            ['1303'] = "nH",
            ['1400'] = "F",
            ['1401'] = "mF",
            ['1402'] = "µF",
            ['1403'] = "nF",
            ['1404'] = "pF",
            ['1500'] = "C",
            ['1501'] = "nC",
            ['1502'] = "pC",
            ['1600'] = "Hz",
            ['1601'] = "kHz",
            ['1602'] = "MHz",
            ['1603'] = "1/s",
            ['1700'] = "radian/s",
            ['1701'] = "U/min",
            ['1702'] = "rpm",
            ['1800'] = "W",
            ['1801'] = "mW",
            ['1802'] = "kW",
            ['1803'] = "MW",
            ['1900'] = "N",
            ['1901'] = "kN",
            ['1902'] = "MN",
            ['1903'] = "kp",
            ['1904'] = "kgf",
            ['1905'] = "lb",
            ['2000'] = "Pa",
            ['2001'] = "bar",
            ['2002'] = "mbar",
            ['2003'] = "kbar",
            ['2004'] = "pas",
            ['2005'] = "hPa",
            ['2006'] = "kPa",
            ['2007'] = "psi",
            ['2008'] = "N/mm²",
            ['2009'] = "N/m²",
            ['2010'] = "N/cm²",
            ['2101'] = "Nm",
            ['2100'] = "J",
            ['2102'] = "kNm",
            ['2103'] = "MNm",
            ['2104'] = "ftlb",
            ['2105'] = "inlb",
            ['2200'] = "Nm",
            ['2201'] = "Nm/radian",
            ['2202'] = "oz-in",
            ['2300'] = "m/m",
            ['2301'] = {"µm/m","microstrain", "µE", "µStrain"},
            ['2302'] = "strain",
            ['2303'] = "mm/m",
            ['2400'] = "m/s",
            ['2401'] = "km/h",
            ['2402'] = "mph",
            ['2403'] = "fps",
            ['2500'] = "m/s²",
            ['2501'] = "ga",
            ['2700'] = "kg/m³",
            ['2701'] = "g/l",
            ['2800'] = "m³/s",
            ['2801'] = "l/min",
            ['2802'] = "m³/h",
            ['2803'] = "gpm",
            ['2804'] = "cfm",
            ['2805'] = "l/h",
            ['2900'] = "%",
            ['2901'] = "%o",
            ['2902'] = "ppm",
            ['3000'] = "%/°C",
            ['3001'] = "%o/°C",
            ['3002'] = "ppm/°C",
            ['3100'] = "Imp",
            ['3101'] = "kImp",
            ['5001'] = "%/decade",
            ['5002'] = "dB",
            ['5003'] = "l/l",
            ['5004'] = "m³/m³",
            ['5005'] = "m³",
            ['5006'] = "mm²",
            ['5007'] = "kg/s",
            ['5008'] = "mole/l",
            ['5009'] = "mole/m³",
            ['5010'] = "N/m",
            ['5011'] = "RH",
            ['5012'] = "V/(m/s²)",
            ['5013'] = "V/C",
            ['5014'] = "V/N",
            ['5015'] = "V/Pa",
            ['5016'] = "W/°C",
            ['100000'] = "UserDefined",
        }, p, fromxml)
end

m.common.calibration_sensor_scaling_mode = m.common.calibration_mode
m.common.calibration_sensor_scaling_units = m.common.units
m.common.calibration_units_conversion_units = m.common.units
m.common.calibration_rezero_units = m.common.units
m.common.calibration_define_source_units_units = m.common.units

return m
