require 'strict'

local ffi = require 'ffi'
local cjson = require 'cjson.safe'
local config = require 'system_config'
local libte = ffi.load(config.test_engine_library)

ffi.cdef[[
    struct cmsg {
        int64_t         timestamp;
        uint32_t        id;
        uint8_t         data[9];
    };

    struct can_signal *canSignalInit(const char *stanza);
]]

local m = {}

-- Read messages from the input pipe and if they are for me, decode, apply
-- scale and offset, sequentialize, and send downstream.
function m.run(node, msg)
    libte.canSignalRun(node, msg)
end

function m.new(stanza, pipe_rate)
    assert(stanza, "Missing parameter 'stanza'")
    assert(pipe_rate, "Missing parameter 'pipe_rate'")

    -- Calculate the aggregate scale and offset
    local slope = 1.0
    local intercept = 0.0
    local csteps = {
        {
            slope = tonumber(stanza.db_scale or 1.0),
            intercept = tonumber(stanza.db_offset or 0.0),
        },
        {
            slope = stanza.scale or 1.0,
            intercept = stanza.offset or 0.0
        },
    }
    for _, cal in ipairs(csteps) do
        slope = slope * cal.slope
        intercept = cal.slope * intercept + cal.intercept
    end
    stanza.scale = slope
    stanza.offset = intercept
    stanza.pipe_rate = pipe_rate
    return libte.canSignalInit(cjson.encode(stanza) or '{}')
end

return m
