-- time_channel.lua - Implements the Time computed channel.

local dbg = require 'dbg'
local ffi = require 'ffi'
local rational = require 'rational'
local ts = require 'timestamps'
local Frame = require 'test_engine/frame'
local Pipe = require 'test_engine/pipe'

local Frame_create = Frame.create
local Frame_clone = Frame.clone
local Pipe_add_frame = Pipe.add_frame
local Pipe_new_pipe_list = Pipe.new_pipe_list

local m = {}

-- Function called to do object's processing
function m:run()
    local current_time = ts.stamp_to_epoch(ts.ts_to_stamp(ts.get_ts()))
    if current_time < self.start_time then
        current_time = 0
    elseif self.duration and current_time > self.start_time + self.duration then
        if self.duration_expired_cb then
            self.duration_expired_cb()
            self.duration_expired_cb = nil
        end
        current_time = self.duration
    else
        if self.acquisition_start_cb then
            self.acquisition_start_cb()
            self.acquisition_start_cb = nil
        end
        current_time = current_time - self.start_time
    end

    -- If no pipes, we are done
    if self.pipe_list[0] == nil then return end

    -- Create a single entry frame
    local root = Frame_create(1, ffi.sizeof('float'))
    if root == nil then return end

    root.hdr.ts = 0

    -- Add the single entry to the frame
    root.count = 1
    local d = ffi.cast('float *', root.buf)
    d[0] = current_time

    -- Add the frame to the pipe(s)
    local p = self.pipe_list[0]
    Pipe_add_frame(p, root)
    while p.next ~= nil do
        Pipe_add_frame(p.next, Frame_clone(root))
        p = p.next
    end
end

-- Setup function
function m:setup(sie_writer)
end

-- Function called to instantiate the object
function m:new(stanza, node_list, sie_index)
    self.stanza = stanza
    self.pipe_list = Pipe_new_pipe_list()
    self.start_time = ts.iso_to_epoch(stanza.start_time)
    self.duration = stanza.duration
    return self, 0
end

return m
