local cjson = require 'cjson.safe'

-- Function used to get the platform information from a device
local function query_platform_info(network, device)
    local rrs = network:QueryRR(device, 'TXT', '*', '*')
    if not rrs then return platform_info end
    for _, rr in ipairs(rrs) do
        local tag, json = rr[3]:match('(.-:) (.+)')
        if tag and json and tag == 'platform_info:' then
            return cjson.decode(json)
        end
    end
    return
end

-- Function used to get the platform information for ALL devices
local function get_platform_info(network)
    local platform_info = {}

    -- First grab a list of devices and their IP addresses
    local rrs = network:QueryRR('*', 'A', '*', '*')
    if not rrs then return platform_info end
    for _, rr in ipairs(rrs) do
        platform_info[rr[1]] = query_platform_info(network, rr[1])
    end
    return platform_info
end

return get_platform_info
